from zope.component import getUtility
from zope.component import adapts
from zope.interface import implements
from zope.interface import Interface
from zope.formlib import form
from zope import schema

from Products.CMFDefault.formlib.schema import SchemaAdapterBase
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView

from plone.app.controlpanel.form import ControlPanelForm
from qi.jabberHelpdesk.interfaces import IHelpdeskSettings
from qi.jabberHelpdesk import HelpdeskMessageFactory as _
from qi.jabberHelpdesk.interfaces import email_constraint

class HelpdeskSettings(BrowserView):

	implements(IHelpdeskSettings)

	def __init__(self, context, request):
		""" init view """
		self.context = context
		self.request = request
		portal_props = getToolByName(context, 'portal_properties')
		self.properties = getattr(portal_props, 'helpdesk_properties', None)

	def _get_property(self,property_id):
		if not self.properties:
			return None
		return getattr(self.properties,property_id)

	@property
	def xmlrpc_host(self):
		return self._get_property('xmlrpc_host')

	@property
	def xmlrpc_port(self):
		return int(self._get_property('xmlrpc_port'))


class IHelpdeskSettingsSchema(Interface):
	"""
	Helpdesk xmlrpc server settings
	"""

	xmlrpc_host = schema.TextLine(title=_(u"Helpdesk XMLRPC server"),
			description=_(u"The address of your local jabber xmlrpc server."),
			default=u'localhost',
			required=True)

	xmlrpc_port = schema.Int(title=_(u"Helpdesk XMLRPC server port "),
			description=_(u"The port of your XMLRPC server"),
			default=6666,
			required=True)


class HelpdeskControlPanel(ControlPanelForm):

	form_fields = form.FormFields(IHelpdeskSettingsSchema)

	label = _("Helpdesk settings")
	description = _("Server settings for the helpdesks of this site.")
	form_name = _("Helpdesk settings")

class HelpdeskControlPanelAdapter(SchemaAdapterBase):

	adapts(IPloneSiteRoot)
	implements(IHelpdeskSettingsSchema)

	def __init__(self, context):
		super(HelpdeskControlPanelAdapter, self).__init__(context)
		properties = getToolByName(context, 'portal_properties')
		self.context = properties.helpdesk_properties


	def get_xmlrpc_host(self):
		return getattr(self.context,'xmlrpc_host',"localhost")

	def set_xmlrpc_host(self, value):
		self.context._updateProperty('xmlrpc_host', value)

	xmlrpc_host = property(get_xmlrpc_host,set_xmlrpc_host)

	def get_xmlrpc_port(self):
		return int(getattr(self.context,'xmlrpc_port',"6667"))

	def set_xmlrpc_port(self, value):
		self.context._updateProperty('xmlrpc_port', value)

	xmlrpc_port = property(get_xmlrpc_port,set_xmlrpc_port)



import re
emoticons = { ':)' : '<img src="++resource++qi.jabberHelpdesk.resources/smiley_smile.png" alt=":)" title="Smile" />'
            , ':(' : '<img src="++resource++qi.jabberHelpdesk.resources/smiley_sad.png" alt=":(" title="Sad" />'
            , '8-)' : '<img src="++resource++qi.jabberHelpdesk.resources/smiley_cool.png" alt="8)" title="Cool" />'
            , ':D' : '<img src="++resource++qi.jabberHelpdesk.resources/smiley_lol.png" alt=":D" title="Big grin" />'
            , ':|' : '<img src="++resource++qi.jabberHelpdesk.resources/smiley_skeptic.png" alt=":|" title="Skeptic" />'
            , ':o' : '<img src="++resource++qi.jabberHelpdesk.resources/smiley_surprised.png" alt=":o" title="Surprised" />'
            , ':P' : '<img src="++resource++qi.jabberHelpdesk.resources/smiley_tongue.png" alt=":P" title="Tongue-in-cheek" />'
            , ';)' : '<img src="++resource++qi.jabberHelpdesk.resources/smiley_wink.png" alt=";)" title="Wink" />'
            , ':-)' : '<img src="++resource++qi.jabberHelpdesk.resources/smiley_smile.png" alt=":)" title="Smile" />'
            , ':-(' : '<img src="++resource++qi.jabberHelpdesk.resources/smiley_sad.png" alt=":(" title="Sad" />'
            , ':-D' : '<img src="++resource++qi.jabberHelpdesk.resources/smiley_lol.png" alt=":D" title="Big grin" />'
            , ':-|' : '<img src="++resource++qi.jabberHelpdesk.resources/smiley_skeptic.png" alt=":|" title="Skeptic" />'
            , ':-o' : '<img src="++resource++qi.jabberHelpdesk.resources/smiley_surprised.png" alt=":o" title="Surprised" />'
            , ':-P' : '<img src="++resource++qi.jabberHelpdesk.resources/smiley_tongue.png" alt=":P" title="Tongue-in-cheek" />'
            , ';-)' : '<img src="++resource++qi.jabberHelpdesk.resources/smiley_wink.png" alt=";)" title="Wink" />'
            }
regex = re.compile("(%s)(?!\")" % "|".join(map(re.escape, emoticons.keys())))

def replaceEmoticons(text):
    """
    """
    return regex.sub(lambda mo, d=emoticons: d[mo.string[mo.start():mo.end()]], text)
    
    
    
    


from zope.interface import implements

from zope.component.factory import Factory
from plone.app.content.interfaces import INameFromTitle
from plone.app.content.item import Item
from qi.jabberHelpdesk.interfaces import IHelpdesk
from qi.jabberHelpdesk import HelpdeskMessageFactory as _
from BTrees.OOBTree import OOSet

class Helpdesk(Item):
	implements(IHelpdesk, INameFromTitle)
	portal_type="Jabber Helpdesk"
	title=u""
	description = u""
	botJid = u""
	botPassword = u""
	persistent = False
	
	def __init__(self,id=None):
		super(Helpdesk,self).__init__(id)

helpdeskFactory = Factory(Helpdesk, title=_(u"Create a new jabber helpdesk"))


from zope.formlib import form
from plone.app.form import base

from Acquisition import aq_inner

from qi.jabberHelpdesk.interfaces import IHelpdeskAgents
from qi.jabberHelpdesk import HelpdeskMessageFactory as _
	
class HelpdeskAgentsEditForm(base.EditForm):
	"""Edit form for projects
	"""
	
	form_fields = form.Fields(IHelpdeskAgents)

	label = _(u"Edit helpdesk agents")
	form_name = _(u"Helpdesk agents")


from qi.jabberHelpdesk.kss.IJabberHelpdeskKSSCommands import IJabberHelpdeskKSSCommands
from kss.core import CommandSet
from zope import interface

class JabberHelpdeskKSSCommands(CommandSet):
	interface.implements(IJabberHelpdeskKSSCommands)
	
	def resetScrollbar(self, selector):
		command = self.commands.addCommand('resetScrollbar', selector)
	def resetInput(self, selector):
		command = self.commands.addCommand('resetInput', selector)



# -*- coding: utf-8 -*-

from zope.interface import Interface
from zope.interface import Attribute

from zope import schema
from qi.jabberHelpdesk import HelpdeskMessageFactory as _
import re

email_constraint = re.compile("^[a-zA-Z][\w\.-]*[a-zA-Z0-9]@[a-zA-Z0-9][\w\.-]*[a-zA-Z0-9]\.[a-zA-Z][a-zA-Z\.]*[a-zA-Z]$").match
class IHelpdesk(Interface):
	"""
	Interface for helpdesk
	"""
	title = schema.TextLine(title=_(u"Title"),
							description=_(u"Name of the helpdesk"),
							required=True)
							
	description = schema.Text(title=_(u"Description"),
							description=_(u"A short summary of the helpdesk"),
							required=True)
	
	botJid = schema.ASCIILine(title=_(u"Bot's JID"),
							description=_(u"The Jabber ID of the bot assigned to this helpdesk"),
							constraint = email_constraint,
							required=True)
	botPassword = schema.Password(title=_(u"Bot's password"),
							description=_(u"The bot's password."),
							default=None,
							required=False)
		
	persistent = schema.Bool(title=_(u"Persistent bot"),
							description = _(u"Specifies whether the bot remains loaded even if nobody is using it"),
							default = False,
							)
class IHelpdeskAgents(Interface):
	"""
	"""
	agentJids = schema.List(title = _(u"Agents IDs"),
							description = _(u"The jabber IDs of auhorized agents, typically of the form userID@jabberserver"),
							value_type=schema.ASCIILine(
								title=_(u"Agent's jabber ID"),
								constraint=email_constraint)
							)
	
class IChatRequest(Interface):
	"""Interface describing a chat request
	"""
	name = schema.ASCIILine(title=_(u"Name"),
									description=_(u"Your full name."),
									required=True)

	subject = schema.ASCIILine(title=_(u"Subject"),
									description=_(u"What you would like to discuss."),
									required=True)


class IHelpdeskSettings(Interface):
	"""
	Helpdesk xmlrpc server settings
	"""

	xmlrpc_host = Attribute("Helpdesk XMLRPC server")
	xmlrpc_port = Attribute("Helpdesk XMLRPC server port ")



class IMessageHandler(Interface):
	"""
	Message handler utility interface
	"""
	
	def userLogin(botID,user,name,subject):
		"""
		"""
	
	def userLogout(botID,user):
		"""
		"""
	
	def sendUserMessage(botID,user,message):
		"""
		"""
		
	def getUserMessages(botID,user):
		"""
		"""
		
	def getAliveAgents(botID):
		"""
		"""

	def getAvailableAgents(botID):
		"""
		"""
		
	def getHelpdeskAgents(botID):
		"""
		"""
		
	def getAgentAvatarB64(botID):
		"""
		"""

	def getAgentVCard(botID,userID):
		"""
		"""

	def loadBot(botID,botPass,persistent):
		"""
		"""
	
	def addAgent(botID,botPass):
		"""
		"""
	
	def addBot(self,botJID,botPass):
		"""
		"""

	def removeBot(self,botJID):
		"""
		"""



from qi.jabberHelpdesk.interfaces import IHelpdesk, IHelpdeskAgents
from zope.component import adapts
from zope.interface import implements
from zope.component import getMultiAdapter


class HelpdeskAgentsEditor(object):
	adapts(IHelpdesk)
	implements(IHelpdeskAgents)
	
	def __init__(self, context):
		self.context = context
		self.mh = getMultiAdapter((self.context,self.context.request),name="helpdesk_xmlrpc")
		self.mh.loadBot(self.context.botJid,self.context.botPassword,self.context.persistent)
		
	def _setAgents(self, value):
		oldAgents = self.mh.getHelpdeskAgents(self.context.botJid)
		toRemove = [agent for agent in oldAgents if agent not in value]
		toAdd = [agent for agent in value if agent not in oldAgents]
		for agent in toRemove:
			self.mh.removeAgent(self.context.botJid,agent)
		for agent in toAdd:
			self.mh.addAgent(self.context.botJid,agent)
 
	def _getAgents(self):
		return self.mh.getHelpdeskAgents(self.context.botJid)
	
	agentJids = property(_getAgents, _setAgents)


from zope.i18nmessageid import MessageFactory
from Products.CMFCore.permissions import setDefaultRoles

HelpdeskMessageFactory = MessageFactory('qi.jabberHelpdesk')
setDefaultRoles("qi.jabberHelpdesk: Add helpdesk", ('Manager',))
setDefaultRoles("qi.jabberHelpdesk: Use helpdesk", ('Anonymous',))

def initialize(context):
	"""Initializer called when used as a Zope 2 product."""
	pass
	


from Acquisition import aq_inner

from zope import schema
from zope.component import getMultiAdapter
from zope.formlib import form
from zope.interface import implements
from zope.component import getUtility

from plone.app.portlets.portlets import base
from plone.app.vocabularies.catalog import SearchableTextSourceBinder
from plone.app.form.widgets.uberselectionwidget import UberMultiSelectionWidget
from plone.memoize.instance import memoize
from plone.memoize import ram
from plone.portlets.interfaces import IPortletDataProvider
from plone.app.portlets.cache import render_cachekey

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from qi.jabberHelpdesk import HelpdeskMessageFactory as _
from qi.jabberHelpdesk.interfaces import IHelpdesk
from Products.Archetypes.atapi import DisplayList
from Products.ATContentTypes.interfaces import IATImage

from qi.jabberHelpdesk.interfaces import IMessageHandler

class IHelpdeskPortlet(IPortletDataProvider):
	"""
	"""
	targetHelpdesk = schema.List(title=_(u"Target helpdesk"),
			description=_(u"Link to the helpdesk of your choice"),
			required=True,
			min_length=1,
			max_length=1,
			value_type=schema.Choice(
				source=SearchableTextSourceBinder({'object_provides' : IHelpdesk.__identifier__},default_query='path:'))
			)

	portletImage = schema.List(title=_(u"Portlet image"),
			description=_(u"An image to be displayed in the body of the portlet"),
			required=True,
			min_length=1,
			max_length=1,
			value_type=schema.Choice(
				source=SearchableTextSourceBinder({'portal_type' : 'Image' },default_query='path:'))
			)
	

class Assignment(base.Assignment):
	"""
	"""
	
	implements(IHelpdeskPortlet)
	def __init__(self,targetHelpdesk=[],portletImage=[]):
		self.targetHelpdesk = targetHelpdesk
		self.portletImage = portletImage

	@property
	def title(self):
		return _(u"Helpdesk portlet")

class Renderer(base.Renderer):

	render = ViewPageTemplateFile('helpdeskPortlet.pt')

	def __init__(self, context, request, view, manager, data):
		"""
		"""
		self.context = context
		self.request = request
		self.data = data
		self.helpdesk = self._getFromPath(self.data.targetHelpdesk[0])
		self.image = self._getFromPath(self.data.portletImage[0])
		self.mh = getMultiAdapter((self.context,self.request),name="helpdesk_xmlrpc") 
		pm = getToolByName(self.context,'portal_membership')
		member = pm.getAuthenticatedMember()
		self.permissionGrant =	member.has_permission('qi.jabberHelpdesk: Use helpdesk',self.context) 

	@property
	def availableAgents(self):
		if self.mh.loadBot(self.helpdesk.botJid,self.helpdesk.botPassword,self.helpdesk.persistent):
			return len(self.mh.getAvailableAgents(self.helpdesk.botJid))
		return 0
	
	@property
	def available(self):
		"""
		"""
		return self.permissionGrant and self.availableAgents
	
	@property	
	def helpdeskLink(self):
		"""
		"""
		return self.helpdesk.absolute_url()
	
	@property
	def helpdeskImage(self):
		"""
		"""
		return self.image.absolute_url()
		
	def _getFromPath(self,path):
		""" gets the ad the path is pointing to"""
		if path.startswith('/'):
			path = path[1:]
		if not path:
			return None
		portal_state = getMultiAdapter((self.context, self.request), name=u'plone_portal_state')
		portal = portal_state.portal()
		return portal.unrestrictedTraverse(path, default=None)

class AddForm(base.AddForm):
	form_fields = form.Fields(IHelpdeskPortlet)
	form_fields['targetHelpdesk'].custom_widget = UberMultiSelectionWidget
	form_fields['portletImage'].custom_widget = UberMultiSelectionWidget
	
	label = _(u"Add helpdesk portlet")
	description = _(u"This portlet lets people ask for online support from the helpdesk.")

	def create(self,data):
		return Assignment(**data)
		#return Assignment(targetHelpdesk=data.get('targetHelpdesk',[]),
		#				portletImage=data.get('portletImage',[]))

class EditForm(base.EditForm):
	form_fields = form.Fields(IHelpdeskPortlet)
	form_fields['targetHelpdesk'].custom_widget = UberMultiSelectionWidget
	form_fields['portletImage'].custom_widget = UberMultiSelectionWidget
	label = _(u"Edit helpdesk portlet")
	description = _(u"This portlet lets people ask for online support from the helpdesk.")


from zope.interface import implements
from zope.component import getMultiAdapter

from qi.jabberHelpdesk.interfaces import IMessageHandler
#from zope.component import getUtility
from xmlrpclib import Server
from Products.Five.browser import BrowserView

class HelpdeskXMLRPC(BrowserView):
	"""
	"""
	implements(IMessageHandler)
	
	def __init__(self, context, request):
		""" init view """
		self.context = context
		self.request = request
		settings =	getMultiAdapter((context,request), name="helpdesk_settings")
		self.rpcserver = Server('http://%s:%i'%(settings.xmlrpc_host,settings.xmlrpc_port))
	def userLogin(self,botID,userID,name,subject):
		"""
		"""
		(result,agent)= self.rpcserver.userLogin(botID,userID,name,subject)
		if result:
			return agent
		raise Exception("No agent available")
			
	def userLogout(self,botID,userID):
		"""
		"""
		self.rpcserver.userLogout(botID,userID)
		
	def sendUserMessage(self,botID,userID,message):
		"""
		"""		
		result= self.rpcserver.sendUserMessage(botID,userID,message)
		if not result:
			raise Exception("You have been disconnected")
			
	def getUserMessages(self,botID,userID):
		"""
		"""
		(result,messages,files) = self.rpcserver.getUserMessages(botID,userID)
		if result:
			return [messages,files]
		raise Exception("You have been disconnected")
	
	def getAliveAgents(self,botID):
		"""
		"""
		return self.rpcserver.getAliveContacts(botID)

	def getAvailableAgents(self,botID):
		"""
		"""
		return self.rpcserver.getAvailableContacts(botID)

	def getHelpdeskAgents(self,botJID):
		"""
		"""
		return self.rpcserver.getContacts(botJID)
		
	def getAgentAvatarB64(self,botID,userID):
		"""
		"""
		return self.rpcserver.getContactAvatarB64(botID,userID)

	def getAgentVCard(self,botID,userID):
		"""
		"""
		return self.rpcserver.getContactVCard(botID,userID)

	def loadBot(self,botID,botPass,persistent):
		"""
		"""
		return self.rpcserver.loadBot(botID,botPass,persistent)
		
	def addAgent(self,botJID,agentId):
		"""
		"""
		return self.rpcserver.addContact(botJID,agentId)
	
	def removeAgent(self,botJID,agentId):
		"""
		"""
		return self.rpcserver.delContact(botJID,agentId)
	
	def addBot(self,botJID,botPass):
		"""
		"""
		return self.rpcserver.addUser(botJID,botPass)

	def removeBot(self,botJID):
		"""
		"""
		return self.rpcserver.delUser(botJID)



from zope.component import getUtility
from Products.CMFCore.utils import getToolByName 
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from qi.jabberHelpdesk import HelpdeskMessageFactory as _


class ChatView(BrowserView):
	"""
	"""
	__call__ = ViewPageTemplateFile('chat.pt')

	def botJid(self):
		"""
		"""
		return self.context.botJid

	def render(self):
		"""Simply return the template
		"""
		return self.template()

	def getMember(self):
		"""
		"""
		self.membership = getToolByName(self.context, 'portal_membership')
		if not self.membership.isAnonymousUser():
			minfo = self.membership.getMemberInfo()
			return minfo['fullname'] or self.membership.getAuthenticatedMember()
		return None
	
	


from zope.component import getUtility, getMultiAdapter
import base64
from cStringIO import StringIO
from Acquisition import aq_inner
import PIL.Image

from DateTime import DateTime
from elementtree.ElementTree import XML as parseET
from Products.CMFCore.utils import getToolByName 
from plone.intelligenttext.transforms import convertHtmlToWebIntelligentPlainText, convertWebIntelligentPlainTextToHtml
from plone.app.kss.plonekssview import PloneKSSView
from random import choice
import string
from qi.jabberHelpdesk.interfaces import IMessageHandler
from qi.jabberHelpdesk.browser.emoticons import replaceEmoticons
from qi.jabberHelpdesk import HelpdeskMessageFactory as _

msgFoot = '<dd class="msgFoot"><span class="msgBL"></span><span class="msgBR"></span></dd></dl>'

class HelpdeskKSSView(PloneKSSView):
	"""
	"""

	def __init__(self, *args):
		super(HelpdeskKSSView, self).__init__(*args)
		self.core = self.getCommandSet('core')
		self.plonekss = self.getCommandSet('plone')
		self.jhkss = self.getCommandSet('jabberHelpdesk')
		self.mh = getMultiAdapter((self.context,self.request),name="helpdesk_xmlrpc")
		self.botJid = self.context.botJid

	def login(self,userName,subject):
		"""
		A user connects to the chat room.
		"""
		# Generate unique userID		
		chars = string.letters + string.digits
		userID = ''.join([choice(chars) for i in range(10)])
		self.core.setStateVar("helpdesk-userID",userID)
		self.core.setStateVar("helpdesk-lastMsgFrom",'')
		self.core.focus("#jabberHelpdeskMsgInput")
		try:
			
			agent=self.mh.userLogin(self.botJid,userID,userName,subject)
			vCard = self.mh.getAgentVCard(self.botJid,userID)
			self.core.setStateVar("helpdesk-lastMsgFrom",'')
			self.core.setStateVar("helpdesk-agent",agent)
			self.plonekss.issuePortalMessage(_(u"You are connected to the helpdesk. Someone will be with you shortly."),'info')
			vCardElem = self.core.getHtmlIdSelector("jabberHelpdeskAgentDetails")
			vCardDetails='<img id="agentAvatar" src="%s/helpdesk_avatar?userID=%s"/>'%(aq_inner(self.context).absolute_url(),userID)
			if vCard['FN'] or vCard['DESC']:
				vCardDetails += "<label>"+_(u"Talking with: ")+"</label>"+'<span id="jabberHelpdeskAgentDesc">'+vCard['FN']+" "+vCard['DESC']+"</span>"
			self.core.insertHTMLAsLastChild(vCardElem, vCardDetails)
		except Exception,Detail:
			return self._connectionError(Detail)
		return self.render()
		
	def logout(self,userID):
		"""
		"""
		self.mh.userLogout(self.botJid,userID)
		return self.render()
		
		
	def ping(self,userID,lastMsgFrom,userName='',message=None):		
		"""
		"""
		messages=[]
		try:
			(supportmessages,files) = self.mh.getUserMessages(self.botJid,userID)
		except Exception,Detail:
			return self._connectionError(Detail)
			
		if message:
			try:
				self.mh.sendUserMessage(self.botJid,userID,message)
				self.jhkss.resetInput("#jabberHelpdeskMsgInput")
				self.core.focus("#jabberHelpdeskMsgInput")	
			except Exception,Detail:
				return self._connectionError(Detail)
			
		self._updateMessages(supportmessages,message,lastMsgFrom,userName,userID,files)
		
		return self.render()
	
	def avatar(self,userID):
		b64 = self.mh.getAgentAvatarB64(self.botJid,userID)
		binary = base64.decodestring(b64)
		response = self.request.response
		response.setHeader('content-type', 'image/png')
		return binary
		
	def refreshAvailableAgents(self):
		aagents = self.mh.getAvailableAgents(self.botJid)
		oagents = self.mh.getAliveAgents(self.botJid)
		aaElem = self.core.getHtmlIdSelector("jabberHelpdeskAvailableAgents")
		oaElem = self.core.getHtmlIdSelector("jabberHelpdeskOnlineAgents")
		self.core.replaceInnerHTML(aaElem,str(len(aagents)))
		self.core.replaceInnerHTML(oaElem,str(len(oagents)))

		return self.render()
		
	def _updateMessages(self,supportmessages,usermessage,lastMsgFrom,userName,userID,files):
		"""
		"""

		messagesElem = self.core.getHtmlIdSelector("jabberHelpdeskMessages")			
		lastMsg = self.core.getHtmlIdSelector("lastMsg")
		if supportmessages or files:
			msgElem = ''
			
			for message in supportmessages:
				message = message.encode('utf-8')				
				message = convertHtmlToWebIntelligentPlainText(message)
				message = convertWebIntelligentPlainTextToHtml(message)
				message = replaceEmoticons(message)
				message = message.decode('utf-8')				
				msgElem =msgElem+ '<dd><span>%s</span></dd>'%(message)
			for (path,name,descr) in files:
				b64 = base64.encodestring(path)
				href = "%s/@@fileDownload?file=%s"%(self.context.absolute_url(),b64)
				msgElem = msgElem + '<dd>You received a file:<a href=%s target="_blank">%s</a> %s</dd>'%(href,name,descr)

			if (lastMsgFrom!='support') and (not usermessage):
				msgHead = '<dl id="lastMsg" class="msgCont"><dt class="msgHead"><span class="msgTL"></span>'
				msgHead = msgHead+'<span class="tile">%s</span><span class="msgTR"></span></dt>'%('support')
				
				self.core.setAttribute(lastMsg,'id','')
				self.core.insertHTMLAsLastChild(messagesElem, msgHead+msgElem+msgFoot)
				self.core.setStateVar('helpdesk-lastMsgFrom','support')
				lastMsgFrom='support'
			else:
				self.core.replaceHTML("#lastMsg .msgFoot",msgElem+msgFoot) 
			self.plonekss.issuePortalMessage('','info')
			
		if usermessage:
			#usermessage = usermessage.encode('utf-8')
			usermessage = convertHtmlToWebIntelligentPlainText(usermessage)
			usermessage = convertWebIntelligentPlainTextToHtml(usermessage)
			usermessage = replaceEmoticons(usermessage)
			usermessage = usermessage.decode('utf-8')
			msgElem = '<dd><span>%s</span></dd>'%(usermessage)

			if lastMsgFrom !='user':
				msgHead = '<dl id="lastMsg" class="msgCont"><dt class="msgHead"><span class="msgTL"></span>'
				msgHead = msgHead +'<span class="tile">%s</span><span class="msgTR"></span></dt>'%(userName)
				self.core.setAttribute(lastMsg,'id','')
				self.core.insertHTMLAsLastChild(messagesElem, msgHead+msgElem+msgFoot)
				self.core.setStateVar('helpdesk-lastMsgFrom','user')
			else:
				self.core.replaceHTML("#lastMsg .msgFoot",msgElem+msgFoot)				
			
		if supportmessages or usermessage or files:
			self.jhkss.resetScrollbar(messagesElem)
	
	def _connectionError(self,Detail):
		"""
		"""
		self.plonekss.issuePortalMessage(_("Connection error"),'error')
		return self.render()
		


from zope.component import getUtility
from zope.formlib import form
from zope.interface import implements
from zope.component import getMultiAdapter


from Acquisition import aq_inner
from Products.Five.formlib import formbase
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage

from plone.app.form import named_template_adapter
from plone.app.form.validators import null_validator
from plone.app.form.interfaces import IPlonePageForm

from qi.jabberHelpdesk.interfaces import IChatRequest
from qi.jabberHelpdesk import HelpdeskMessageFactory as _
from qi.jabberHelpdesk.interfaces import IMessageHandler

chatRequestFormView = named_template_adapter(ViewPageTemplateFile('chatRequest.pt'))

class ChatRequestView(formbase.PageForm):
	"""Request a chat
	"""
	implements(IPlonePageForm)
	
	label = _(u"Chat with us")
	description = _(u"Welcome to our helpdesk")

	form_name =_(u"Please fill in your details")
	form_fields = form.FormFields(IChatRequest)
	
	def __init__(self,context,request):
		ChatRequestView.label = context.title
		ChatRequestView.description = context.description
		super(ChatRequestView,self).__init__(context,request)
		self.mh = getMultiAdapter((context,request),name="helpdesk_xmlrpc")
		
	def availableAgents(self):
		if self.mh.loadBot(self.context.botJid,self.context.botPassword,self.context.persistent):
			return (len(self.mh.getAvailableAgents(self.context.botJid)),
					len(self.mh.getAliveAgents(self.context.botJid)),)
		return (0,0,0)
		
	@form.action(_(u"Submit"))
	def action_submit(self, action, data):
		"""
		"""
		
		context = aq_inner(self.context)
		name=''
		subject=''
		try:
			name = data['name']
			subject = data['subject']

		except Exception, e:
			IStatusMessage(self.request).addStatusMessage(e.error_message, type='error')
		
		try:
			if self.mh.loadBot(context.botJid,context.botPassword,context.persistent):
				if self.availableAgents():
					self.request.response.redirect('@@chatView?name=%s&subject=%s'%(name,subject))
					return ''
				else:
					IStatusMessage(self.request).addStatusMessage(_(u"No available agents."), type='error')
			else:
				IStatusMessage(self.request).addStatusMessage(_(u"Could not connect to helpdesk server"), type='error')
		except Exception,e:
			IStatusMessage(self.request).addStatusMessage(_(u"Could not connect to helpdesk server"), type='error')
			print e
	# The null_validator ensures that we can cancel the form even if some
	# required fields are not entered
	@form.action(_(u"Cancel"), validator=null_validator)
	def action_cancel(self, action, data):
		"""Cancel the subscription
		"""
		context = aq_inner(self.context)
		
		confirm = _(u"Chat request cancelled.")
		IStatusMessage(self.request).addStatusMessage(confirm, type='info')
		
		self.request.response.redirect(context.absolute_url())
		return ''


import mimetypes
import os.path
import base64
from Products.Five.browser import BrowserView
from ZPublisher.Iterators import IStreamIterator

class FileIterator(object):
	"""ZPublisher iterator for existing already opened."""
	__implements__ = (IStreamIterator,)

	def __init__(self, input, streamsize=1<<16):
		self.input=input
		self.streamsize=streamsize

	def next(self):
		data = self.input.read(self.streamsize)
		if not data:
			raise StopIteration
		return data

	def __len__(self):
		cur_pos = self.input.tell()
		self.input.seek(0, 2)
		size = self.input.tell()
		self.input.seek(cur_pos, 0)

		return size

class FileDownloadView(BrowserView):
	"""
	"""
	
	def __call__(self):
		path = self.request.get('file',None)
		if not path:
			return ''
		path = base64.decodestring(path)
		filename = path.split('/')[-1]
		output = open(path,"rb")
		iterator=FileIterator(output)

		self.request.response.setHeader(
			'Content-Disposition',
			'attachment; filename=%s'%filename)
		self.request.response.setHeader("Content-Type", self.mimetype(path))
		self.request.response.setHeader('Content-Length', len(iterator))

		return iterator
		

	def mimetype(self,path):
		try:
			return mimetypes.types_map[os.path.splitext(path)[1]]
		except KeyError:
			return "application/octet-stream"



from zope.component import createObject
from zope.formlib import form

from plone.app.form import base

from Acquisition import aq_inner

from qi.jabberHelpdesk.interfaces import IHelpdesk
from qi.jabberHelpdesk import HelpdeskMessageFactory as _

		
		
class HelpdeskAddForm(base.AddForm):
	"""Add form for jabber helpdesks
	"""
	
	form_fields = form.Fields(IHelpdesk)
	
	label = _(u"Add jabber Helpdesk")
	form_name = _(u"Helpdesk settings")
		
	def create(self, data):
		helpdesk = createObject(u"qi.jabberHelpdesk.Helpdesk")
		form.applyChanges(helpdesk, self.form_fields, data)
		return helpdesk
	
class HelpdeskEditForm(base.EditForm):
	"""Edit form for projects
	"""
	
	form_fields = (form.Fields(IHelpdesk).select('title','description') +
					form.Fields(IHelpdesk, for_display=True).select('botJid','botPassword','persistent'))
	
	label = _(u"Edit jabber Helpdesk")
	form_name = _(u"Helpdesk settings")